@@ -316,6 +316,7 @@
      (display (make-random-string size) port))))
 
 (define (create-file name . lines)
+  (catch #f (unlink name))
   (letfd ((fd (open name (logior O_WRONLY O_CREAT O_BINARY) #o600)))
     (let ((port (fdopen fd "wb")))
       (for-each (lambda (line) (display line port) (newline port))
@@ -337,6 +338,8 @@
 	       "no-secmem-warning"
 	       "no-permission-warning"
 	       "batch"
+               "no-auto-key-retrieve"
+               "no-auto-key-locate"
 	       "allow-weak-digest-algos"
 	       (if have-opt-always-trust
 		   "no-auto-check-trustdb" "#no-auto-check-trustdb")
@@ -348,7 +351,10 @@
 	       "allow-preset-passphrase"
 	       "no-grab"
 	       "enable-ssh-support"
+	       (if (flag "--extended-key-format" *args*)
+		   "enable-extended-key-format" "#enable-extended-key-format")
 	       (string-append "pinentry-program " (tool 'pinentry))
+	       (string-append "scdaemon-program " (tool 'scdaemon))
 	       ))
 
 ;; Initialize the test environment, install appropriate configuration
@@ -447,7 +453,7 @@
 	      (with-home-directory gnupghome
 				   (stop-agent)))))
   (catch (log "Warning: Creating socket directory failed:" (car *error*))
-	 (call-popen `(,(tool 'gpgconf) --create-socketdir) ""))
+	 (gpg-conf '--create-socketdir))
   (call-check `(,(tool 'gpg-connect-agent) --verbose
 		,(string-append "--agent-program=" (tool 'gpg-agent)
 				"|--debug-quick-random")
@@ -456,9 +462,33 @@
 ;; Stop the agent and other daemons and remove the socket dir.
 (define (stop-agent)
   (log "Stopping gpg-agent...")
-  (call-check `(,(tool 'gpgconf) --kill all))
+  (gpg-conf '--kill 'all)
   (catch (log "Warning: Removing socket directory failed.")
-	 (call-popen `(,(tool 'gpgconf) --remove-socketdir) "")))
+	 (gpg-conf '--remove-socketdir)))
+
+;; Get the trust level for KEYID.  Any remaining arguments are simply
+;; passed to GPG.
+;;
+;; This function only supports keys with a single user id.
+(define (gettrust keyid . args)
+  (let ((trust
+	  (list-ref (assoc "pub" (gpg-with-colons
+				   `(,@args
+				      --list-keys ,keyid))) 1)))
+    (unless (and (= 1 (string-length trust))
+		 (member (string-ref trust 0) (string->list "oidreqnmfuws-")))
+	    (fail "Bad trust value:" trust))
+    trust))
+
+;; Check that KEYID's trust level matches EXPECTED-TRUST.  Any
+;; remaining arguments are simply passed to GPG.
+;;
+;; This function only supports keys with a single user id.
+(define (checktrust keyid expected-trust . args)
+  (let ((trust (apply gettrust `(,keyid ,@args))))
+    (unless (string=? trust expected-trust)
+	    (fail keyid ": Expected trust to be" expected-trust
+		   "but got" trust))))
 
 
 ;; end
