@@ -105,7 +105,8 @@ static ARGPARSE_OPTS opts[] =
     { oBuilddir, "build-prefix", 2, "@" },
     { oNull, "null", 0, "@" },
     { oNoVerbose, "no-verbose",  0, "@"},
-    {0}
+
+    ARGPARSE_end(),
   };
 
 
@@ -291,7 +292,7 @@ query_swdb (estream_t out, const char *name, const char *current_version)
       goto leave;
     }
   if (!strcmp (name, "gnupg"))
-    search_name = "gnupg21";
+    search_name = GNUPG_SWDB_TAG;
   else if (!strcmp (name, "gnupg1"))
     search_name = "gnupg1";
   else
